import qml
import numpy as mp

def max_size(geom_path, geom_files):
    N_file=len(geom_files)
    N_at=[]
    for i_geom in range(N_file):
        xyz_file=geom_path+'/'+geom_files[i_geom]
        mol=qml.Compound(xyz=xyz_file)
        N_at.append( len(mol.nuclear_charges) )
    max_size = max(N_at) 
    return max_size



from .read_files import read_files
from .bin_spectra import bin_spectra_uniform
from .qml_representation import qml_fchl_rep
from .qml_utils import max_size
from .linalg_solve import linalg_solve
from .plot_spectrum import plot_stem
from .plot_spectrum import plot_smooth
from .plot_spectrum import plot_stem_smooth
from .predict import predict
from .shuffle import shuffle
from .prepare_trainingdata import prepare_trainingdata



import qml
import numpy as np
import qmlspectrum

def qml_fchl_rep(geom_path, read_X, file_X, cut_distance):

    geom_files=qmlspectrum.read_files(geom_path)

    max_size=qmlspectrum.max_size(geom_path, geom_files)
    print('Max. number of atoms in the dataset is ',max_size)

    if read_X:
        print('Loading FCHL representation from ', file_X)
        X = np.load(file_X)
    else:
        print('Calculating FCHL representation')
        N_file=len(geom_files)
        i_file=0
        X=[]
        for i_geom in range(N_file):
            if np.mod(i_file, 100) == 0:
                print(i_file,' molecules out of ', N_file, ' done')
            xyz_file=geom_path+'/'+geom_files[i_geom]
            mol=qml.Compound(xyz=xyz_file)
            representation=qml.fchl.generate_representation(mol.coordinates, mol.nuclear_charges, max_size=max_size, cut_distance=cut_distance)
            X.append(representation)
            i_file=i_file+1
        np.save(file_X, X)
        print('FCHL representation is stored in ', file_X)

    N_mol = len(X)
    return N_mol, max_size, X



import numpy as np
import qmlspectrum
import pandas as pd
'''
A module containing functions for various types of binning.
'''
def bin_spectra_uniform(spec_path, read_P, file_P, wavelength_min, wavelength_max, N_bin):
    '''
    A function for binning spectra using a uniform bin width
    '''
    spec_files=qmlspectrum.read_files(spec_path)
    if read_P:
        dlambda = (wavelength_max - wavelength_min)/N_bin
        lam=[]
        for i_bin in range(N_bin):
            lam.append( (i_bin+1.0/2.0)*dlambda )
        Int_lam = np.load(file_P)

    else:
        print('binning spectra')

        dlambda = (wavelength_max - wavelength_min)/N_bin
        lambda_min=[]
        lambda_max=[]
        lam=[]
        for i_bin in range(N_bin):
            lambda_min.append( (i_bin)*dlambda )
            lambda_max.append( (i_bin+1)*dlambda )
            lam.append( (i_bin+1.0/2.0)*dlambda )
        Int_lam=[]
        N_file=len(spec_files)
        i_file=0
        for spec_csv in spec_files:
            if np.mod(i_file, 100) == 0:
                print(i_file,' out of ', N_file, ' done')
            spec_data=pd.read_csv(spec_path+'/'+spec_csv, names=['wavelength_nm', 'osc_strength'])
            wavelength=spec_data['wavelength_nm']
            f=spec_data['osc_strength']
            sum_f=np.zeros(N_bin)
            for i_bin in range(N_bin):
                scr=f[(wavelength > lambda_min[i_bin] ) & (wavelength <= lambda_max[i_bin])]
                sum_f[i_bin]=np.sum( scr )
            i_file=i_file+1
            Int_lam.append(sum_f)
        Int_lam=np.array(Int_lam)
        np.save(file_P, Int_lam)
        print('data saved in ', file_P)

    return lam, Int_lam




import numpy as np
import random

def shuffle(load_indices, file_indices, N_mol):

    if load_indices:
        indices=np.zeros(N_mol,dtype=int)
        f=open(file_indices, 'r')
        indices=[]
        for line in f:
            string=line.split()[0]
            indices.append(int(string))
        f.close()
    else:
        indices=list( random.sample(range(0,N_mol), N_mol) )
        f = open(file_indices, 'w')
        for i_mol in range(N_mol):
            f.write("%06d"%(indices[i_mol])+'\n')
        f.close()

    return indices
